@@ -12,7 +12,7 @@ class TestConsoleLog(base.BaseNamespacedTestCase):
         super(TestConsoleLog, self).setUp()
         self.net = self.test_client.allocate_network(
             '192.168.242.0/24', True, True, '%s-net-one' % self.namespace)
-        self._await_network_ready(self.net['uuid'])
+        self._await_networks_ready([self.net['uuid']])
 
     def test_console_log(self):
         # Start our test instance
@@ -26,7 +26,7 @@ class TestConsoleLog(base.BaseNamespacedTestCase):
             [
                 {
                     'size': 8,
-                    'base': 'cirros',
+                    'base': 'sf://upload/system/cirros',
                     'type': 'disk'
                 }
             ], None, None)
