@@ -1,5 +1,6 @@
 
 from functools import partial
+import json
 import time
 
 from shakenfist import constants
@@ -7,6 +8,7 @@ from shakenfist import db
 from shakenfist import etcd
 from shakenfist import exceptions
 from shakenfist import logutil
+from shakenfist.util import general as util_general
 
 
 LOG, _ = logutil.setup(__name__)
@@ -24,10 +26,21 @@ class DatabaseBackedObject(object):
     STATE_DELETE_WAIT = 'delete_wait'
     STATE_ERROR = 'error'
 
-    def __init__(self, object_uuid, version=None):
+    ACTIVE_STATES = set([STATE_INITIAL,
+                         STATE_CREATING,
+                         STATE_CREATED,
+                         STATE_ERROR,
+                         STATE_DELETE_WAIT
+                         ])
+
+    def __init__(self, object_uuid, version=None, in_memory_only=False):
         self.__uuid = object_uuid
         self.__version = version
 
+        self.__in_memory_only = in_memory_only
+        if self.__in_memory_only:
+            self.__in_memory_values = {}
+
         self.log = LOG.with_object(self)
 
     @property
@@ -45,16 +58,60 @@ class DatabaseBackedObject(object):
         return (self.object_type, self.__uuid)
 
     def add_event(self, operation, phase, duration=None, msg=None):
-        db.add_event(
-            self.object_type, self.__uuid, operation, phase, duration, msg)
+        if not self.__in_memory_only:
+            db.add_event(
+                self.object_type, self.__uuid, operation, phase, duration, msg)
+
+    @classmethod
+    def from_db(cls, object_uuid):
+        if not object_uuid:
+            return None
+
+        static_values = cls._db_get(object_uuid)
+        if not static_values:
+            return None
+
+        return cls(static_values)
+
+    @classmethod
+    def from_db_by_ref(cls, object_ref, namespace=None):
+        if object_ref and util_general.valid_uuid4(object_ref):
+            # Already a valid UUID
+            return cls.from_db(object_ref)
+
+        # Check if valid name of an active object
+        filters = [partial(state_filter, cls.ACTIVE_STATES)]
+        if namespace:
+            filters.append(partial(namespace_filter, namespace))
+
+        found_obj = None
+        for o in cls.filter(filters):
+            if o.name == object_ref:
+                if found_obj:
+                    raise exceptions.MultipleObjects(
+                        'multiple %ss have the name "%s"' % (
+                            cls.object_type, object_ref))
+                found_obj = o
+
+        return found_obj
+
+    @classmethod
+    def filter(cls, filters):
+        for _, o in etcd.get_all(cls.object_type, None):
+            obj = cls(o)
+            if all([f(obj) for f in filters]):
+                yield obj
 
     @classmethod
     def _db_create(cls, object_uuid, metadata):
+        # NOTE(mikal): the copy() here is important. The logging framework mucks
+        # with the metadata dictionary it is passed in such a way that object
+        # creation breaks. Only give it a copy!
+        LOG.with_fields(metadata.copy()).with_field(
+            cls.object_type, object_uuid).debug('Object created')
         metadata['uuid'] = object_uuid
         etcd.create(cls.object_type, None, object_uuid, metadata)
-        LOG.with_fields({
-            'object_type': cls.object_type
-        }).with_fields(metadata).debug('Object created')
+
         db.add_event(
             cls.object_type, object_uuid, 'db record created', None, None, None)
 
@@ -71,24 +128,42 @@ class DatabaseBackedObject(object):
                 'Unknown version - %s: %s' % (cls.object_type, o))
         return o
 
+    # We need to force in memory values through JSON because some values require
+    # a serializer to run to work when we read them.
     def _db_get_attribute(self, attribute):
-        retval = etcd.get('attribute/%s' % self.object_type,
-                          self.__uuid, attribute)
+        if self.__in_memory_only:
+            retval = json.loads(self.__in_memory_values.get(attribute, 'null'))
+        else:
+            retval = etcd.get('attribute/%s' % self.object_type,
+                              self.__uuid, attribute)
         if not retval:
             return {}
         return retval
 
     def _db_get_attributes(self, attribute_prefix):
-        for key, data in etcd.get_all('attribute/%s' % self.object_type,
-                                      self.__uuid, prefix=attribute_prefix):
-            yield key, data
+        if self.__in_memory_only:
+            for key in self.__in_memory_values.keys():
+                if key.startswith(attribute_prefix):
+                    yield key, json.loads(self.__in_memory_values[key])
+        else:
+            for key, data in etcd.get_all('attribute/%s' % self.object_type,
+                                          self.__uuid, prefix=attribute_prefix):
+                yield key, data
 
     def _db_set_attribute(self, attribute, value):
-        etcd.put('attribute/%s' % self.object_type,
-                 self.__uuid, attribute, value)
+        if self.__in_memory_only:
+            self.__in_memory_values[attribute] = json.dumps(
+                value, indent=4, sort_keys=True, cls=etcd.JSONEncoderCustomTypes)
+        else:
+            etcd.put('attribute/%s' % self.object_type,
+                     self.__uuid, attribute, value)
 
     def _db_delete_attribute(self, attribute):
-        etcd.delete('attribute/%s' % self.object_type, self.__uuid, attribute)
+        if self.__in_memory_only and attribute in self.__in_memory_values:
+            del self.__in_memory_values[attribute]
+        else:
+            etcd.delete('attribute/%s' %
+                        self.object_type, self.__uuid, attribute)
 
     def get_lock(self, subtype=None, ttl=60, relatedobjects=None, log_ctx=None,
                  op=None, timeout=constants.ETCD_ATTEMPT_TIMEOUT):
@@ -182,14 +257,7 @@ def state_filter(states, o):
 
 # Do not use these filters for instances or nodes, use the more
 # specific ones instead
-active_states_filter = partial(
-    state_filter, [DatabaseBackedObject.STATE_INITIAL,
-                   DatabaseBackedObject.STATE_CREATING,
-                   DatabaseBackedObject.STATE_CREATED,
-                   DatabaseBackedObject.STATE_ERROR,
-                   DatabaseBackedObject.STATE_DELETE_WAIT])
-inactive_states_filter = partial(
-    state_filter, [DatabaseBackedObject.STATE_DELETED])
+active_states_filter = partial(state_filter, DatabaseBackedObject.ACTIVE_STATES)
 
 
 def state_age_filter(delay, o):
