@@ -7,7 +7,6 @@ from prometheus_client import start_http_server
 
 from shakenfist.daemons import daemon
 from shakenfist.config import config
-from shakenfist import db
 from shakenfist import etcd
 from shakenfist import logutil
 from shakenfist.util import general as util_general
@@ -75,13 +74,39 @@ def _get_stats():
             ksm_details['memory_ksm_%s' % ent] = int(f.read().rstrip())
     retval.update(ksm_details)
 
-    # Disk info
-    s = os.statvfs(config.STORAGE_PATH)
+    # Disk info. There could be more than one filesystem here, so we track
+    # all of the paths we're fond of.
+    fsids = []
+    minimum = -1
+    total = 0
+    used = 0
+
+    for path in ['', 'blobs', 'image_cache', 'instances', 'uploads']:
+        # We need to make the paths we check if they don't exist, otherwise
+        # they wont be included in the metrics and things get confused.
+        fullpath = os.path.join(config.STORAGE_PATH, path)
+        os.makedirs(fullpath, exist_ok=True)
+        s = os.statvfs(fullpath)
+        free = s.f_frsize * s.f_bavail
+
+        if s.f_fsid not in fsids:
+            total += s.f_frsize * s.f_blocks
+            used += s.f_frsize * (s.f_blocks - s.f_bfree)
+            if minimum == -1 or free < minimum:
+                minimum = free
+
+        if path == '':
+            path = 'sfroot'
+        retval['disk_free_%s' % path] = free
+
+    retval.update({
+        'disk_total': total,
+        'disk_free': minimum,
+        'disk_used': used
+    })
+
     disk_counters = psutil.disk_io_counters()
     retval.update({
-        'disk_total': s.f_frsize * s.f_blocks,
-        'disk_free': s.f_frsize * s.f_bavail,
-        'disk_used': s.f_frsize * (s.f_blocks - s.f_bfree),
         'disk_read_bytes': disk_counters.read_bytes,
         'disk_write_bytes': disk_counters.write_bytes,
     })
@@ -153,9 +178,10 @@ class Monitor(daemon.Daemon):
 
     def run(self):
         LOG.info('Starting')
-        gauges = {'updated_at': Gauge('updated_at',
-                                      'The last time metrics were updated')
-                  }
+        gauges = {
+            'updated_at': Gauge('updated_at',
+                                'The last time metrics were updated')
+        }
 
         last_metrics = 0
 
@@ -168,10 +194,17 @@ class Monitor(daemon.Daemon):
                     gauges[metric] = Gauge(metric, '')
                 gauges[metric].set(stats[metric])
 
-            db.update_metrics_bulk(stats)
+            etcd.put(
+                'metrics', config.NODE_NAME, None,
+                {
+                    'fqdn': config.NODE_NAME,
+                    'timestamp': time.time(),
+                    'metrics': stats
+                },
+                ttl=120)
             gauges['updated_at'].set_to_current_time()
 
-        while True:
+        while not self.exit.is_set():
             try:
                 jobname, _ = etcd.dequeue('%s-metrics' % config.NODE_NAME)
                 if jobname:
@@ -180,7 +213,7 @@ class Monitor(daemon.Daemon):
                         last_metrics = time.time()
                     etcd.resolve('%s-metrics' % config.NODE_NAME, jobname)
                 else:
-                    time.sleep(0.2)
+                    self.exit.wait(0.2)
 
                 timer = time.time() - last_metrics
                 if timer > config.SCHEDULER_CACHE_TIMEOUT:
