@@ -1,8 +1,6 @@
 from functools import partial
-from flask_jwt_extended import get_jwt_identity
-from flask_jwt_extended import jwt_required
-from flask_restful import fields
-from flask_restful import marshal_with
+from flask_jwt_extended import jwt_required, get_jwt_identity
+from flask_restful import fields, marshal_with
 import ipaddress
 
 from shakenfist.config import config
@@ -54,18 +52,18 @@ def _delete_network(network_from_db, wait_interfaces=None):
 
 class NetworkEndpoint(api_base.Resource):
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
-    def get(self, network_uuid=None, network_from_db=None):
+    def get(self, network_ref=None, network_from_db=None):
         return network_from_db.external_view()
 
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
     @api_base.requires_namespace_exist
     @api_base.redirect_to_network_node
-    def delete(self, network_uuid=None, network_from_db=None, namespace=None):
-        if network_uuid == 'floating':
+    def delete(self, network_ref=None, network_from_db=None, namespace=None):
+        if network_ref == 'floating':
             return api_base.error(403, 'you cannot delete the floating network')
 
         n = net.Network.from_db(network_from_db.uuid)
@@ -93,6 +91,9 @@ class NetworkEndpoint(api_base.Resource):
 
         _delete_network(network_from_db)
 
+        # Return UUID in case API call was made using object name
+        return {'uuid': network_from_db.uuid}
+
 
 class NetworksEndpoint(api_base.Resource):
     @marshal_with({
@@ -107,15 +108,16 @@ class NetworksEndpoint(api_base.Resource):
     })
     @jwt_required
     def get(self, all=False):
-        filters = [partial(baseobject.namespace_filter, get_jwt_identity())]
-        if not all:
-            filters.append(baseobject.active_states_filter)
+        with etcd.ThreadLocalReadOnlyCache():
+            filters = [partial(baseobject.namespace_filter, get_jwt_identity()[0])]
+            if not all:
+                filters.append(baseobject.active_states_filter)
 
-        retval = []
-        for n in net.Networks(filters):
-            # This forces the network through the external view rehydration
-            retval.append(n.external_view())
-        return retval
+            retval = []
+            for n in net.Networks(filters):
+                # This forces the network through the external view rehydration
+                retval.append(n.external_view())
+            return retval
 
     @jwt_required
     @api_base.requires_namespace_exist
@@ -130,10 +132,10 @@ class NetworksEndpoint(api_base.Resource):
                                   suppress_traceback=True)
 
         if not namespace:
-            namespace = get_jwt_identity()
+            namespace = get_jwt_identity()[0]
 
         # If accessing a foreign name namespace, we need to be an admin
-        if get_jwt_identity() not in [namespace, 'system']:
+        if get_jwt_identity()[0] not in [namespace, 'system']:
             return api_base.error(
                 401, 'only admins can create resources in a different namespace')
 
@@ -155,7 +157,7 @@ class NetworksEndpoint(api_base.Resource):
         if confirm is not True:
             return api_base.error(400, 'parameter confirm is not set true')
 
-        if get_jwt_identity() == 'system':
+        if get_jwt_identity()[0] == 'system':
             if not isinstance(namespace, str):
                 # A client using a system key must specify the namespace. This
                 # ensures that deleting all networks in the cluster (by
@@ -163,9 +165,9 @@ class NetworksEndpoint(api_base.Resource):
                 return api_base.error(400, 'system user must specify parameter namespace')
 
         else:
-            if namespace and namespace != get_jwt_identity():
+            if namespace and namespace != get_jwt_identity()[0]:
                 return api_base.error(401, 'you cannot delete other namespaces')
-            namespace = get_jwt_identity()
+            namespace = get_jwt_identity()[0]
 
         networks_del = []
         networks_unable = []
@@ -194,17 +196,17 @@ class NetworksEndpoint(api_base.Resource):
 
 class NetworkEventsEndpoint(api_base.Resource):
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
-    def get(self, network_uuid=None, network_from_db=None):
-        return list(db.get_events('network', network_uuid))
+    def get(self, network_ref=None, network_from_db=None):
+        return list(db.get_events('network', network_from_db.uuid))
 
 
 class NetworkInterfacesEndpoint(api_base.Resource):
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
-    def get(self, network_uuid=None, network_from_db=None):
+    def get(self, network_ref=None, network_from_db=None):
         out = []
         for ni in networkinterface.interfaces_for_network(self.network):
             out.append(ni.external_view())
@@ -213,61 +215,57 @@ class NetworkInterfacesEndpoint(api_base.Resource):
 
 class NetworkMetadatasEndpoint(api_base.Resource):
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
-    def get(self, network_uuid=None, network_from_db=None):
-        md = db.get_metadata('network', network_uuid)
+    def get(self, network_ref=None, network_from_db=None):
+        md = db.get_metadata('network', network_from_db.uuid)
         if not md:
             return {}
         return md
 
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
-    def post(self, network_uuid=None, key=None, value=None, network_from_db=None):
-        return api_util.metadata_putpost('network', network_uuid, key, value)
+    def post(self, network_ref=None, key=None, value=None, network_from_db=None):
+        return api_util.metadata_putpost('network', network_from_db.uuid, key, value)
 
 
 class NetworkMetadataEndpoint(api_base.Resource):
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
-    def put(self, network_uuid=None, key=None, value=None, network_from_db=None):
-        return api_util.metadata_putpost('network', network_uuid, key, value)
+    def put(self, network_ref=None, key=None, value=None, network_from_db=None):
+        return api_util.metadata_putpost('network', network_from_db.uuid, key, value)
 
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
-    def delete(self, network_uuid=None, key=None, network_from_db=None):
+    def delete(self, network_ref=None, key=None, network_from_db=None):
         if not key:
             return api_base.error(400, 'no key specified')
 
-        with db.get_lock('metadata', 'network', network_uuid, op='Network metadata delete'):
-            md = db.get_metadata('network', network_uuid)
+        with db.get_lock('metadata', 'network', network_from_db.uuid, op='Network metadata delete'):
+            md = db.get_metadata('network', network_from_db.uuid)
             if md is None or key not in md:
                 return api_base.error(404, 'key not found')
             del md[key]
-            db.persist_metadata('network', network_uuid, md)
+            db.persist_metadata('network', network_from_db.uuid, md)
 
 
 class NetworkPingEndpoint(api_base.Resource):
     @jwt_required
-    @api_base.arg_is_network_uuid
+    @api_base.arg_is_network_ref
     @api_base.requires_network_ownership
     @api_base.redirect_to_network_node
     @api_base.requires_network_active
-    def get(self, network_uuid=None, address=None, network_from_db=None):
-        ipm = IPManager.from_db(network_uuid)
+    def get(self, network_ref=None, address=None, network_from_db=None):
+        ipm = IPManager.from_db(network_from_db.uuid)
         if not ipm.is_in_range(address):
             return api_base.error(400, 'ping request for address outside network block')
 
-        n = net.Network.from_db(network_uuid)
-        if not n:
-            return api_base.error(404, 'network %s not found' % network_uuid)
-
         out, err = util_process.execute(
             None, 'ip netns exec %s ping -c 10 %s' % (
-                network_uuid, address),
+                network_from_db.uuid, address),
             check_exit_code=[0, 1])
         return {
             'stdout': out,
